'==========================================================================
'
' NAME: InstallClient.vbs
'
' PUBLISHER: upKeeper Solution AB
' AUTHOR: Peter Johansson, Hi5 AB
' DATE: 2012-09-20
'
' UPDATE
' 2022-08-30 PeJo, upKeeper Solutions AB - Changed file search and version check
' 2013-12-09 PeJo, Hi5 AB - Run in own process 
'
' COMMENT: Upgrade or install latest upkeeper client
'
' ERROR CODES:
'   -1 - MSI installatiopnfile missing 
'  300 - Client cant be upgraded, to old.
' 1619 - newer client already installed
'==========================================================================
Option Explicit

Const NEWVERSION = "4.11.2"

'"MsiExec.exe /I %MSIFILE% ALLUSERS=1 ENDPOINT1=https://client.upkeeper.se /qn /l*v C:\Windows\Temp\upkeeper4.log"

Const INSTALLSTRING = "MsiExec.exe /I ""%MSIFILE%"" ALLUSERS=1 ENDPOINT1=https://client.upkeeper.se ENDPOINT2=https://client.upkeeper.se /qn /l*v C:\Windows\Temp\upkeeper4.log" 
Const UPGRADESTRING = "MsiExec.exe /I ""%MSIFILE%"" /qn"
Const REPAIRSTRING = "MsiExec.exe /F ""%MSIFILE%"" /qn"
Const LOGPATH = "C:\Windows\Temp\InstallClient.log"


' Script should not be edited below this line

Const MINVERSION = "3.1.2"

Const FORREADING = 1
Const FORWRITING = 2
Const FORAPPENDING = 8

Dim wshShell
Dim wshNetwork
Dim objFSO 
Dim objLogfile

Set wshShell = CreateObject("WScript.Shell")
Set wshNetwork = CreateObject("WScript.Network")
Set objFSO = CreateObject("Scripting.FileSystemObject")

Main()


Function GetVersion(filepath)

	Dim version

	version = objFSO.GetFileVersion(filepath)

	GetVersion = version

End Function


Function WriteLog(message)

	objLogFile.WriteLine GetDate("") & " " & GetTime(":") & " - " & message

End Function 

Function GetMSIFile(strPath, bol64bit)

	Dim objFolder
	Dim objFiles
	Dim objFile
	Dim strArcitecture

	strArcitecture = "86"

	if(bol64bit) Then strArcitecture = "64"

	Set objFolder = objFSO.GetFolder(strPath)
	Set objFiles = objFolder.Files

	For Each objFile in objFiles
		If (instr(UCASE(objFile.Name), ".MSI") > 0) AND (instr(objFile.Name, strArcitecture) > 0) Then
			WriteLog "Found proper installfile: " & objFile.Name  
			GetMSIFile = objFile.Name
			Exit Function
		End if
	Next

	WriteLog "No proper MSI file found!"

End Function


Function GetDate(divider)

	Dim strDate

		
	strDate = DatePart("YYYY", Now) & Right("0" & DatePart("M", Now),2) & Right("0" & DatePart("D", Now),2)


	GetDate = strDate

End Function

Function GetTime(divider)

	Dim strTime
	
	strTime = Right("0" & DatePart("h", Now),2) & divider & Right("0" & DatePart("n", Now),2) & divider & Right("0" & DatePart("s", Now),2) 

	GetTime = strTime

End Function

Function FirstVersionValueISHigher(strFirstVersionValue, strSecondVersionValue)

	Dim strFirstValue
	Dim strSecondValue
	Dim i
	Dim intResult

	WriteLog("Is version " & strFirstVersionValue & " higher then version " & strSecondVersionValue & ".")

	'Split the two supplied file versions by the "." character
	strFirstValue = Split(strFirstVersionValue, ".")
	strSecondValue = Split(strSecondVersionValue, ".")

	For i = 0 To UBound(strFirstValue)

		' First is higher
		If CDbl(strFirstValue(i)) > CDbl(strSecondValue(i)) Then
			WriteLog("First value is higher when comparing index " & i & " (" & strFirstValue(i) & " > " & strSecondValue(i) & ")")   
			intResult = 1
		
		' Second is higher
		ElseIf CDbl(strFirstValue(i)) < CDbl(strSecondValue(i)) Then
			WriteLog("Second value is higher when comparing index " & i & " (" & strFirstValue(i) & " < " & strSecondValue(i) & ")")   
			intResult = -1

		End If

		'If we have found that the result is not > or <, no need to proceed
		If intResult <> 0 Then Exit For

	Next

	if(intResult=0) Then WriteLog("First and second value are equal. (" & strFirstValue(i) & " = " & strSecondValue(i) & ")")


	FirstVersionValueISHigher = intResult

End Function



Function InstallClient(strFile)

	Dim strPath
	Dim strVersion
	Dim intExitCode

	strPath = GetInstalledLocation()

	if(Len(strPath)>1) Then
		WriteLog ("Found installed upkeeper client. Path : " & strPath)
		strVersion = GetVersion(strpath)
		if (FirstVersionValueISHigher(MINVERSION, strVersion)<>-1) Then
			WriteLog("Current version of upKeeper can not be upgraded. Current version : " & strVersion & " (Upgradable from : " & MINVERSION & ")")
			InstallClient = 300
			Exit function
		End if
		if(FirstVersionValueISHigher(strVersion, NEWVERSION)=1 OR FirstVersionValueISHigher(strVersion, NEWVERSION)=0) Then
			WriteLog("No need to upgrade. Installed version: " & strVersion & " (New version: " & NEWVERSION & ")")
			InstallClient = 0
			Exit function
		End If
		
		'Upgrade
		WriteLog ("Current client can be upgraded. Current version: " & strVersion & " (" & NEWVERSION & ") Initalize upgrade.")
		WriteLog ("Execute: " & Replace(INSTALLSTRING,"%MSIFILE%", strFile))
		intExitCode = wshShell.RUn(Replace(UPGRADESTRING,"%MSIFILE%",strFile) ,1 ,True)
		WriteLog ("Upgrade executed with exitcode: " & intExitcode)
	Else
		'Install
		WriteLog ("Could not find any installed upkeeper client. Initalize new installation.")
		WriteLog ("Execute: " & Replace(INSTALLSTRING,"%MSIFILE%", strFile)) 	
		intExitCode = wshShell.RUn(Replace(INSTALLSTRING,"%MSIFILE%", strFile) ,1 ,True)
		WriteLog ("Installation executed with exitcode: " & intExitcode)
		

	End If
		
	InstallClient = intExitCode	

End Function


Function Repair(strFile)
	WriteLog ("Could not find upkeeper service. Initalize repair.")
	WriteLog ("Execute: " & Replace(REPAIRSTRING,"%MSIFILE%", strFile)) 	
	intExitCode = wshShell.RUn(Replace(REPAIRSTRING,"%MSIFILE%", strFile) ,1 ,True)
	WriteLog ("Repair executed with exitcode: " & intExitcode)
End Function

Function GetInstalledLocation()

	Dim strProgramFilesPath	
	Dim objFolder

	strProgramFilesPath = wshShell.ExpandEnvironmentStrings("%ProgramFiles%")

	Set objFolder = objFSO.GetFolder(strProgramFilesPath)

	GetInstalledLocation = GetupKeeperClientFile(objFolder)	

End Function


Function ServiceIsInstalled()

	Dim strComputer
	Dim objWMIService
	Dim colRunningServices
	Dim IsInstalled

	strComputer = "." 
	Set objWMIService = GetObject("winmgmts:" _ 
	& "{impersonationLevel=impersonate}!\\" & strComputer & "\root\cimv2") 
 
	Set colRunningServices = objWMIService.ExecQuery("Select * from Win32_Service Where Name='upkeeperClientService'")  

	IsInstalled = False

	If(colRunningServices.Count > 0) Then
		IsInstalled = True 
	End If
	
	ServiceIsInstalled = IsInstalled

End Function


Function GetupKeeperClientFile(strBaseFolder)

	Dim objFolder
	Dim objSubFolders
	Dim objFiles
	Dim objFile
	Dim strFile


	WriteLog "Search subfolders of folder: " & strBaseFolder
	Set objFolder = objFSO.GetFolder(strBaseFolder)
	Set objSubFOlders = objFolder.SubFolders

	For Each objFolder in objSubFolders
		If(Instr(lcase(objfolder), "upkeeper") > 0) Then
			WriteLog "Search for clientfile in folder: " & objFolder
			Set objFiles = objFolder.Files
			For Each objFile in objFiles
				WriteLog "Check file: " & objFile & " Client pos: " & instr(lcase(objFile),"client.manager") & " EXE pos: " & instr(lcase(objFile),".exe")
				if((instr(lcase(objFile),"client.manager") > 0) AND (instr(lcase(objFile),".exe") > 0)) Then
					strFile = Replace(objFile, objFolder & "\", "")
					WriteLog("Found file: " & strFile &  " Path: " & objFile & " Position: " & instr(lcase(strFile),"upkeeper") ) 
					If(instr(lcase(strFile),"upkeeper") > 0) Then
						GetupKeeperClientFile = objFile
						Exit Function
					End If					
				End if

			Next

			'Check subfolders to current folder
			strFile = GetupKeeperClientFile(objFolder)
			If(len(strFile)>0) Then
				
				GetupKeeperClientFile = strFile
				Exit Function
			End If 						
		End If
	Next	

End Function



Private Function Is64bit()
	Dim bol64bit 
	Dim strArcitecture	

	bol64bit = False
	strArcitecture = "32"

	If objFSO.FolderExists("C:\Program Files (x86)") Then 
		bol64bit = True
		strArcitecture = "64"
	End If
	
	WriteLog "Arctitecture : " & strArcitecture & "-bit" 

	is64bit = bol64bit
End Function


Function Main()

	Dim strCurrentFolder
	Dim bol64bit
	Dim strFile	
	Dim intExitCode	
	Dim strTempDir
	Dim copyResult 

	intExitCode = -1

	'Init log	
	Set objLogfile = objFSO.OpenTextFile(LOGPATH, FORAPPENDING, True)

	WriteLog("Script started - " & wscript.scriptname)
	WriteLog("Computer name: " & wshNetwork.Computername)

	'Get working folder
	strCurrentFolder = replace(wscript.scriptfullname,wscript.scriptname, "")

	'Get environment 32- or 64-bit
	bol64bit = Is64bit		

	'Get MSI files
	strFile = GetMSIFile(strCurrentFolder, bol64bit)

	If(len(strFile) > 0) Then
		' Install client or upgrade if needed
		intExitCode = InstallClient(strFile)

		If(ServiceIsInstalled() = False) Then
			Repair(strFile)
		End If
	End If
	


	WriteLog("Script ended")

	objLogfile.Close
	Set objLogFile = Nothing
	Set objFSO = Nothing

	WScript.Quit(intExitCode)

End Function